/* Dashboard */
div#collective-sylvester-what-are-you-doing {
    color: #333333;
    font-size: 1.5em;
    float: left;
    vertical-align: bottom;
}

div#collective-sylvester-characters-remaining {
    color: #cccccc;
    float: right;
    font-weight: bolder;
    font-size: 2.45em;
}

form#collective-sylvester-dashboard-tweet-form {
    margin: 0;
}
form#collective-sylvester-dashboard-tweet-form textarea {
    border: solid 1px #aaaaaa;
    padding: 4px 2px;
    width: 99%;
    margin: 12px 0px 12px 0px;
}    

form#collective-sylvester-dashboard-tweet-form div.latest {
    float: left;
    color: #333333;
}    

form#collective-sylvester-dashboard-tweet-form div.buttons {
    float: right;
}    

form#collective-sylvester-dashboard-tweet-form div.buttons input {
    background-image: url(++resource++collective.sylvester.round_button.gif);
    color: #aaaaaa;
    border: 0;
    font-size: 14px;
    height: 32px;
    width: 115px;
}    

div.collective-sylvester-dashboard-navigation a {
    float: right;
    text-decoration: none;
    padding-left: 16px;
    margin-left: 8px;
}    

div.collective-sylvester-dashboard-navigation a.selected {
    float: left;
    color: #333333;
    font-size: 1.5em;  
    margin-left: 0px;
}    

div.collective-sylvester-dashboard-navigation a.mainfeed {
    background-image: url(++resource++collective.sylvester.mainfeed.png);
    background-repeat: no-repeat;
}

div.collective-sylvester-dashboard-navigation a.friends {
    background-image: url(++resource++collective.sylvester.friends.png);
    background-repeat: no-repeat;
}

div.collective-sylvester-dashboard-navigation a.replies {
    background-image: url(++resource++collective.sylvester.replies.png);
    background-repeat: no-repeat;
}



/* Statuslet containers */
ul.collective-sylvester-statuses li {
    margin: 0px 0px 0px -16px;
    padding: 8px; 
    list-style-image: none;
    list-style-type: none;
    border-top: dashed 1px #d2dada;    
}  

li.collective-sylvester-statuses-highlight {
    background-color: #f7f7f7;
}    

/* Statuslet styles */
table.collective-sylvester-statuslet {
    font-size: 1.2em;
    color: #333333;
    width: 100%;
    line-height: 130%;
}    

table.collective-sylvester-statuslet tr td {
    vertical-align: top;
}    

table.collective-sylvester-statuslet tr td.image {
    width: 48px;
    padding-right: 5px;
}

table.collective-sylvester-statuslet tr td.actions {
    text-align: right;
    width: 20px;
    padding-left: 5px;
}

a.collective-sylvester-reply {
    background-image: url(++resource++collective.sylvester.icon_reply.gif);
    background-position: 50% 50%;
    background-repeat: no-repeat;
    border-bottom: 0 !important;
    padding: 3px 8px;
    text-decoration: none;
}

div.collective-sylvester-tweet-time-and-source {
    font-style: italic;
    font-size: 0.9em;
    color: #999999;    
}
